#include <fggl/data/model.hpp>

#include <map>
#include <stdexcept>

using namespace fggl::data;

Mesh::Mesh() : m_verts(0), m_index(0) {
}

void Mesh::pushIndex(unsigned int idx) {
	assert( idx < m_verts.size() || idx == this->restartVertex );
	m_index.push_back(idx);
}

unsigned int Mesh::pushVertex(Vertex vert) {
	unsigned int idx = m_verts.size();
	m_verts.push_back( vert );
	return idx;
}

int Mesh::indexOf(Vertex vert) {
	int i = 0;
	for ( auto& vert_v : m_verts ) {
		if ( vert.posititon == vert_v.posititon ) {
			return i;
		}
		++i;
	}
	return -1;
}

void Mesh::removeDups() {
	// lookups to make detecting duplicates easier
	std::map<Vertex, unsigned int> mapping;
	unsigned int nextID = 0;

	// new data
	std::vector< Vertex > newVerts;
	std::vector< unsigned int > newIndexes;
	newIndexes.reserve( m_index.size() ); // newIndex will be same size as oldIndex

	for ( auto& idx : m_index ) {
		auto& vertex = m_verts[idx];
		auto newID = nextID;
		try {
			newID = mapping.at( vertex );
		} catch ( std::out_of_range& e ){
			mapping[ vertex ] = newID;
			newVerts.push_back( vertex );
			++nextID;
		}
		newIndexes.push_back( newID );
	}

	// FIXME must be faster way to do this...
	m_verts.clear();
	m_index.clear();

	for ( auto& vert : newVerts ) {
		m_verts.push_back( vert );
	}

	for ( auto& idx : newIndexes ) {
		m_index.push_back( idx );
	}

	// ensure we're not wasting space
	m_verts.shrink_to_fit();
	m_index.shrink_to_fit();
}

