
#include <fggl/gfx/atlas.hpp>

#include <fggl/math/types.hpp>
#include <array>
#include <vector>
#include <string>

#define STBRP_STATIC
#define STB_RECT_PACK_IMPLEMENTATION
#include "../stb/stb_rect_pack.h"

using Query = std::vector<fggl::gfx::Bounds2D>;

static void populate_stbrp_query(Query& query, std::vector<stbrp_rect>& data) {
    for ( std::size_t i = 0; i < query.size(); i++ ) {
        data.push_back({
            (int)i,
            query[i].size.x,
            query[i].size.y,
            0,
            0,
            0
        });
    }
}

static void unpack_stbrp_query(Query& query, std::vector<stbrp_rect>& data) {
    for( const auto& rect : data ) {
        query[ rect.id ].pos = { rect.x, rect.y };
    }
}

bool pack_iter(int width, int height, std::vector<stbrp_rect>& query) {
    stbrp_node* tmp = new stbrp_node[width];

    // setup context
    stbrp_context context;
    stbrp_init_target( &context, width, height, tmp, width );

    // see if it worked
    auto result = stbrp_pack_rects( &context, query.data(), query.size() );
    return result == 1;
}

namespace fggl::gfx {


    bool pack(std::vector<Bounds2D>& pack){
        // setup query structure
        std::vector<stbrp_rect> query;
        query.reserve( pack.size() );
        populate_stbrp_query(pack, query);

        // try packing into powers of 2, starting with 32, up to 4096
        for ( int i=5; i <= 12; i++ ) {
            int dim = i * i;
            if ( pack_iter(dim, dim, query) ) {
                unpack_stbrp_query(pack, query);
                return true;
            }
        }

        return false;
    }

} // namespace fggl::gfx

