#ifndef FGGL_DEBUG_H
#define FGGL_DEBUG_H

#include <utility>
#include <functional>
#include <unordered_map>

#include <fggl/gfx/window.hpp>

namespace fggl::debug {

	using DebugUIDraw = std::function<void(bool *)>;

	struct DebugWindow {
		bool m_visible;
		DebugUIDraw m_callback;
	};

	class DebugUI {
		public:
			explicit DebugUI(std::shared_ptr<gfx::GlfwWindow> &window);
			~DebugUI();

			void frameStart();
			void draw();

			inline void addWindow(const std::string &name, DebugUIDraw window) {
				m_windows[name] = DebugWindow{true, std::move(window) };
			}

			inline void visible(bool state) {
				m_visible = state;
			}

			inline bool visible() const {
				return m_visible;
			}

		private:
			bool m_visible;
			std::unordered_map<std::string, DebugWindow> m_windows;
	};

}

#endif
