#ifndef FGGL_GFX_ATLAS_H
#define FGGL_GFX_ATLAS_H

#include <fggl/math/types.hpp>
#include <vector>

namespace fggl::gfx {

	struct Bounds2D {
		glm::ivec2 pos;
		glm::ivec2 size;
	};
	bool pack(std::vector<Bounds2D> &bounds);

	template<typename T>
	class ImageAtlas {
		public:
			struct SubImage {
				const T name;
				Bounds2D bounds;
			};

			SubImage &locate(int id) const;
			SubImage &locate(const T &name) const;

			static ImageAtlas<T> *pack(std::vector<SubImage> &images) {
				// extract bounds
				std::vector<Bounds2D> bounds;
				for (auto &image : images) {
					bounds.push_back(image.bounds);
				}

				// allocate the bounds back
				auto result = gfx::pack(bounds);
				for (std::size_t i = 0; i < images.size(); i++) {
					images[i].bounds = bounds[i];
				}

				return new ImageAtlas<T>(images);
			}

		private:
			ImageAtlas(const std::vector<SubImage> &images) : m_images(images) {
			}

			std::vector<SubImage> m_images;
	};

};

#endif
