#ifndef FGGL_GFX_SHADER_H
#define FGGL_GFX_SHADER_H

#include <cstdio>
#include <fggl/gfx/ogl/backend.hpp>
#include <fggl/data/storage.hpp>

#include <filesystem>
#include <stdexcept>
#include <unordered_map>

namespace fggl::gfx {

	struct ShaderConfig {
		// required
		std::string name;
		std::string vertex;
		std::string fragment;

		// optional parts
		std::string geometry;
		bool hasGeom = false;
	};

	inline ShaderConfig ShaderFromName(const std::string &name) {
		return {
			name,
			name + "_vert.glsl",
			name + "_frag.glsl"
		};
	}

	struct BinaryCache {
		void *data = nullptr;
		GLsizei size = 0;
		GLenum format = GL_INVALID_ENUM;
	};

	class ShaderCache {
		public:
			ShaderCache(std::shared_ptr<fggl::data::Storage> storage);
			~ShaderCache() = default;

			GLuint load(const ShaderConfig &config);
			GLuint getOrLoad(const ShaderConfig &config);

			GLuint get(const std::string &name);

		private:
			std::shared_ptr<fggl::data::Storage> m_storage;
			std::unordered_map<std::string, GLuint> m_shaders;

			// opengl operations
			bool compileShader(const std::string &, GLuint);

			// file io operations
			bool loadFromDisk(GLuint pid, const ShaderConfig &config);
			void saveToDisk(GLuint pid, const ShaderConfig &config);

			bool m_binary;
			void cacheSave(GLuint pid, BinaryCache *cache);
			bool cacheLoad(GLuint pid, const BinaryCache *cache);
	};

}

#endif
