
#ifndef FGGL_GFX_WINDOWING_H
#define FGGL_GFX_WINDOWING_H

#include <memory>
#include <fggl/gfx/paint.hpp>
#include "fggl/ecs3/ecs.hpp"

namespace fggl::gfx {

	struct Bounds {
		float top;
		float right;
		float bottom;
		float left;
	};

	class Graphics {
		public:
			virtual ~Graphics() = default;
			virtual void clear() = 0;
			virtual void resize(int width, int height) = 0;

			virtual Bounds canvasBounds() = 0;
			virtual void draw2D(const Paint &paint) = 0;

			virtual void drawScene(ecs3::World&) = 0;
	};

	class Window {
		public:
			virtual ~Window() = default;
			virtual void activate() const = 0;

			template<typename T>
			void make_graphics() {
				activate();
				m_graphics = std::make_unique<T>(*this);
			}

			// window-related getters
			[[nodiscard]]
			virtual math::vec2i frameSize() const = 0;

			[[nodiscard]]
			virtual bool wantClose() const = 0;

			// window callbacks
			virtual void frameStart() = 0;
			virtual void frameEnd() = 0;

			Graphics &graphics() {
				return *m_graphics;
			}

		protected:
			std::unique_ptr<Graphics> m_graphics;
	};

	class WindowService {
		public:
			Window &create();
	};

} // namespace fggl::gfx

#endif
