#ifndef FGGL_GUI_CONTAINERS_H
#define FGGL_GUI_CONTAINERS_H

#include <fggl/gui/widget.hpp>

namespace fggl::gui {

	class Container : public Widget {
		public:
			Container() = default;
			virtual ~Container() = default;

			inline void clear() {
				m_children.clear();
			}

			void add(std::unique_ptr<Widget> widget);
			virtual inline void layout() {}

			bool contains(const math::vec2 &point) override;
			Widget *getChildAt(const math::vec2 &point) override;
			void render(gfx::Paint &paint) override;

		private:
			bool m_dirty;

		protected:
			std::vector<std::unique_ptr<Widget>> m_children;
	};

	class Panel : public Container {
		public:
			Panel() = default;
			~Panel() = default;

			void render(gfx::Paint& paint) override;

	};

	enum class LayoutAxis {
		LINE_AXIS,
		PAGE_AXIS
	};

	/**
	 * A box-like container.
	 * This container lines child elements up along an axis.
	 *
	 * This is a fairly common approach in may layout frameworks, for example:
	 *   * Java's BoxLayout
	 *   * GTK's Boxes
	 *   * QT's QBoxLayout
	 *   * Android's XML box things
	 */
	/*class Box : public Container {
		public:
			Box(LayoutAxis axis);
			void layout() override;
		private:
			const LayoutAxis m_axis;
	};*/

}; //namespace fggl::gui

#endif
