#ifndef FGGL_INPUT_KEYBOARD_H
#define FGGL_INPUT_KEYBOARD_H

#include <array>
#include <vector>
#include <unordered_set>
#include <iostream>

namespace fggl::input {
	using scancode_t = int;

	class KeyboardState {
		public:
			KeyboardState() = default;

			inline void clear() {
				m_keys.clear();
			}

			inline void set(scancode_t code, bool state) {
				if (state) {
					m_keys.insert(code);
				} else {
					m_keys.erase(code);
				}
			}

			inline bool down(scancode_t scancode) const {
				if (m_keys.empty())
					return false;

				return m_keys.count(scancode) > 0;
			}

		private:
			std::unordered_set<scancode_t> m_keys;
	};

	class KeyboardInput {
		public:
			inline void frame(float dt) {
				m_prev = m_curr;
			}

			inline void set(scancode_t scancode, bool state) {
				m_curr.set(scancode, state);
			}

			inline bool down(scancode_t scancode) const {
				return m_curr.down(scancode);
			}

			inline bool pressed(scancode_t scancode) const {
				return m_curr.down(scancode) && !m_prev.down(scancode);
			}

			inline bool released(scancode_t scancode) const {
				return !m_curr.down(scancode) && m_prev.down(scancode);
			}

		private:
			KeyboardState m_curr;
			KeyboardState m_prev;
	};

};

#endif
