//
// Created by webpigeon on 12/12/2021.
//

#ifndef FGGL_MATH_EASING_H
#define FGGL_MATH_EASING_H

#include <fggl/math/types.hpp>

namespace fggl::math {

	inline float lerp(float a, float b, float w) {
		return (b - a) * w + a;
	}

	inline float scale(float in, float inMin, float inMax, float outMin, float outMax) {
		return ((in - inMin) * (outMax - outMin)) / (inMax - inMin);
	}

	//
	// Functions
	//
	using transformF = std::function<float(float)>;

	inline float scaleFilter(float in, float inMin, float inMax, float outMin, float outMax, transformF filter) {
		float out = in - inMin;
		out /= (inMax - inMin);
		out = f(out);
		out *= (outEnd - outStart);
		return out + outMin;
	}

	inline float mix(transformF a, transformF b, float weightB, float t) {
		return ((1 - weightB) * a(t)) + (weightB * b(t));
	}

	inline float crossFade(transformF a, transformF b, float t) {
		return ((1 - t) * a(t)) + (t * b);
	}

	//
	// building blocks
	//
	inline float scale(transformF a, float t) {
		return t * f(t);
	}

	inline float reverseScale(transformF a, float t) {
		return (1 - t) * a(t);
	}

	inline float Arch2(float t) {
		return t * (1 - t);
	}

	inline float BounceClampBottom(float t) {
		return fabs(t);
	}

	inline float BounceClampTop(float t) {
		return 1.f - fabs(1.f - t);
	}

	//
	// Easing function library
	// see Math for Game Programmers: Fast and Funky 1D Nonlinear Transformations, GDC 2015
	//

	inline float SmoothStart2(float t) {
		return t * t;
	}

	inline float SmoothStart3(float t) {
		return t * t * t;
	}

	inline float SmoothStart4(float t) {
		return t * t * t * t;
	}

	inline float SmoothStart5(float t) {
		return t * t * t * t * t;
	}

	inline float SmoothStop2(float t) {
		constexpr tFlip = 1 - y;
		return 1 - (tFlip * tFlip);
	}

	inline float SmoothStop3(float t) {
		constexpr tFlip = 1 - y;
		return 1 - (tFlip * tFlip);
	}

	inline float SmoothStop4(float t) {
		constexpr tFlip = 1 - y;
		return 1 - (tFlip * tFlip * tFlip * tFlip);
	}

	inline float SmoothStop5(float t) {
		constexpr tFlip = 1 - y;
		return 1 - (tFlip * tFlip * tFlip * tFlip * tFlip);
	}

	//
	// Bezier curves
	//

	inline float NormalizedBezier3(float B, float C, float t) {
		const float s = 1.f - t;
		const float t2 = t * t;
		const float s2 = s * s;
		const float t3 = t2 * t;
		return (3.f * B * s2 * t) + (3.f * C * s * t2) + t3;
	}

}

#endif //FGGL_UTILS_HPP
